#ifndef MSP_GL_MODULE_H_
#define MSP_GL_MODULE_H_

#include <string>
#include <msp/io/base.h>
#include "glsl/compiler.h"
#include "glsl/sourcemap.h"

namespace Msp {
namespace GL {

class Resources;

class Module
{
public:
	enum Format
	{
		GLSL,
	};

protected:
	Module() { }
public:
	virtual ~Module() { }

	virtual Format get_format() const = 0;

	void set_source(const std::string &);
	void load_source(IO::Base &, Resources *, const std::string &);
	void load_source(IO::Base &, const std::string &);
private:
	virtual void compile(SL::Compiler &) = 0;
};

class GlslModule: public Module
{
private:
	std::string prepared_source;
	SL::SourceMap source_map;

public:
	virtual Format get_format() const { return GLSL; }

private:
	virtual void compile(SL::Compiler &);

public:
	const std::string &get_prepared_source() const { return prepared_source; }
	const SL::SourceMap &get_source_map() const { return source_map; }
};

} // namespace GL
} // namespace Msp

#endif
